@@ -1,7 +1,7 @@
 /* mailbox.h -- Mailbox format definitions
- $Id: mailbox.h,v 1.71 2003/01/08 17:40:17 rjs3 Exp $
+ * $Id: mailbox.h,v 1.77.2.11 2005/05/27 17:40:54 ken3 Exp $
  *
- * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -43,9 +43,14 @@
 #define INCLUDED_MAILBOX_H
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <limits.h>
+#include <config.h>
 
 #include "auth.h"
+#include "quota.h"
+#include "message_uuid.h"
+
 
 #define BIT32_MAX 4294967295U
 
@@ -59,6 +64,10 @@ typedef unsigned short bit32;
 #error dont know what to use for bit32
 #endif
 
+#ifdef HAVE_LONG_LONG_INT
+typedef unsigned long long int bit64;
+#endif
+
 #define MAX_MAILBOX_NAME 490
 #define MAX_MAILBOX_PATH 4096
 
@@ -71,29 +80,17 @@ typedef unsigned short bit32;
 #define MAILBOX_FORMAT_NORMAL	0
 #define MAILBOX_FORMAT_NETNEWS	1
 
-#define MAILBOX_MINOR_VERSION	4
+#define MAILBOX_MINOR_VERSION	7
+#define MAILBOX_CACHE_MINOR_VERSION 2
 
 #define FNAME_HEADER "/cyrus.header"
 #define FNAME_INDEX "/cyrus.index"
 #define FNAME_CACHE "/cyrus.cache"
 #define FNAME_SQUAT_INDEX "/cyrus.squat"
-#define FNAME_QUOTADIR "/quota/"
+#define FNAME_EXPUNGE_INDEX "/cyrus.expunge"
 
 #define MAILBOX_FNAME_LEN 256
 
-#define QUOTA_UNITS (1024)
-
-struct quota {
-    int fd;
-
-    int lock_count;
-    char *root;
-
-    /* Information in quota file */
-    unsigned long used;
-    int limit;			/* in QUOTA_UNITS */
-};
-
 struct mailbox {
     int header_fd;
     int index_fd;
@@ -112,14 +109,15 @@ struct mailbox {
     int seen_lock_count;
     int pop_lock_count;
 
-    unsigned long header_ino;
+    ino_t header_ino;
     time_t index_mtime;
-    unsigned long index_ino;
+    ino_t index_ino;
     off_t index_size;
 
     /* Information in mailbox list */
     char *name;
     char *path;
+    char *mpath;
     char *acl;
     long myrights;
 
@@ -137,7 +135,7 @@ struct mailbox {
     unsigned long exists;
     time_t last_appenddate;
     unsigned long last_uid;
-    unsigned long quota_mailbox_used;
+    uquota_t quota_mailbox_used;
     unsigned long pop3_last_login;
     unsigned long uidvalidity;
 
@@ -147,11 +145,18 @@ struct mailbox {
     int dirty;
 
     int pop3_new_uidl;
+    unsigned long leaked_cache_records;
 
     /* future expansion -- won't need expand the header */
-    unsigned long spares[4];
+    unsigned long spares[2];
 
     struct quota quota;
+
+    /* Information in current session */
+    int examining;	/* Nonzero if opened with EXAMINE command */
+    int keepingseen;	/* Nonzero if /Seen is meaningful */
+    unsigned allseen;	/* Last UID if all msgs /Seen last checkpoint */
+    unsigned recentuid;	/* UID of last non-\Recent message */
 };
 
 struct index_record {
@@ -165,9 +170,12 @@ struct index_record {
     time_t last_updated;
     bit32 system_flags;
     bit32 user_flags[MAX_USER_FLAGS/32];
+    unsigned long content_lines;
+    unsigned long cache_version;
+    struct message_uuid uuid;
 };
 
-/* Offsets of index header fields */
+/* Offsets of index/expunge header fields */
 #define OFFSET_GENERATION_NO 0
 #define OFFSET_FORMAT 4
 #define OFFSET_MINOR_VERSION 8
@@ -176,19 +184,19 @@ struct index_record {
 #define OFFSET_EXISTS 20
 #define OFFSET_LAST_APPENDDATE 24
 #define OFFSET_LAST_UID 28
-#define OFFSET_QUOTA_MAILBOX_USED 32
-#define OFFSET_POP3_LAST_LOGIN 36
-#define OFFSET_UIDVALIDITY 40
-#define OFFSET_DELETED 44	/* added for ACAP */
-#define OFFSET_ANSWERED 48
-#define OFFSET_FLAGGED 52
-#define OFFSET_POP3_NEW_UIDL 56	/* added for Outlook stupidity */
-#define OFFSET_SPARE0 60
-#define OFFSET_SPARE1 64
-#define OFFSET_SPARE2 68
-#define OFFSET_SPARE3 72
+#define OFFSET_QUOTA_MAILBOX_USED64 32  /* offset for 64bit quotas */
+#define OFFSET_QUOTA_MAILBOX_USED 36    /* offset for 32bit quotas */
+#define OFFSET_POP3_LAST_LOGIN 40
+#define OFFSET_UIDVALIDITY 44
+#define OFFSET_DELETED 48      /* added for ACAP */
+#define OFFSET_ANSWERED 52
+#define OFFSET_FLAGGED 56
+#define OFFSET_POP3_NEW_UIDL 60	/* added for Outlook stupidity */
+#define OFFSET_LEAKED_CACHE 64 /* Number of leaked records in cache file */
+#define OFFSET_SPARE1 68
+#define OFFSET_SPARE2 72
 
-/* Offsets of index_record fields in index file */
+/* Offsets of index_record fields in index/expunge file */
 #define OFFSET_UID 0
 #define OFFSET_INTERNALDATE 4
 #define OFFSET_SENTDATE 8
@@ -199,24 +207,48 @@ struct index_record {
 #define OFFSET_LAST_UPDATED 28
 #define OFFSET_SYSTEM_FLAGS 32
 #define OFFSET_USER_FLAGS 36
+#define OFFSET_CONTENT_LINES (OFFSET_USER_FLAGS+MAX_USER_FLAGS/8) /* added for nntpd */
+#define OFFSET_CACHE_VERSION OFFSET_CONTENT_LINES+sizeof(bit32)
+#define OFFSET_MESSAGE_UUID OFFSET_CACHE_VERSION+sizeof(bit32)
 
-#define INDEX_HEADER_SIZE (OFFSET_SPARE3+4)
-#define INDEX_RECORD_SIZE (OFFSET_USER_FLAGS+MAX_USER_FLAGS/8)
+#define INDEX_HEADER_SIZE (OFFSET_SPARE2+sizeof(bit32))
+#define INDEX_RECORD_SIZE (OFFSET_MESSAGE_UUID+MESSAGE_UUID_PACKED_SIZE)
+
+/* Number of fields in an individual message's cache record */
+#define NUM_CACHE_FIELDS 10
 
 #define FLAG_ANSWERED (1<<0)
 #define FLAG_FLAGGED (1<<1)
 #define FLAG_DELETED (1<<2)
 #define FLAG_DRAFT (1<<3)
 
-extern char *mailbox_cache_header_name[];
-extern int mailbox_num_cache_header;
+struct mailbox_header_cache {
+    const char *name; /* Name of header */
+    bit32 min_cache_version; /* Cache version it appeared in */
+};
 
-typedef int mailbox_decideproc_t(struct mailbox *mailbox, 
-				 void *rock, char *indexbuf);
+#define MAX_CACHED_HEADER_SIZE 32 /* Max size of a cached header name */
+extern const struct mailbox_header_cache mailbox_cache_headers[];
+extern const int MAILBOX_NUM_CACHE_HEADERS;
+
+/* Bitmasks for expunging */
+enum {
+    EXPUNGE_FORCE =		(1<<0),
+    EXPUNGE_CLEANUP =		(1<<1)
+};
+
+int mailbox_cached_header(const char *s);
+int mailbox_cached_header_inline(const char *text);
+
+unsigned long mailbox_cache_size(struct mailbox *mailbox, unsigned msgno);
+
+typedef int mailbox_decideproc_t(struct mailbox *mailbox, void *rock,
+				 char *indexbuf, int expunge_flags);
 
 typedef void mailbox_notifyproc_t(struct mailbox *mailbox);
 
 extern void mailbox_set_updatenotifier(mailbox_notifyproc_t *notifyproc);
+extern mailbox_notifyproc_t *mailbox_get_updatenotifier(void);
 
 extern int mailbox_initialize(void);
 
@@ -226,26 +258,30 @@ extern char *mailbox_message_fname(struct mailbox *mailbox,
 /* 'len(out) >= MAILBOX_FNAME_LEN' */
 extern void mailbox_message_get_fname(struct mailbox *mailbox,
 				      unsigned long uid,
-				      char *out);
+				      char *out, size_t size);
 
-extern int mailbox_map_message(struct mailbox *mailbox,
-				  int iscurrentdir,
-				  unsigned long uid,
+extern int mailbox_map_message(struct mailbox *mailbox, unsigned long uid,
 				  const char **basep, unsigned long *lenp);
 extern void mailbox_unmap_message(struct mailbox *mailbox,
 				  unsigned long uid,
 				  const char **basep, unsigned long *lenp);
 
 extern void mailbox_reconstructmode(void);
+
+extern int mailbox_stat(const char *mbpath, const char *metapath,
+			struct stat *header, struct stat *index,
+			struct stat *cache);
+
 extern int mailbox_open_header(const char *name, struct auth_state *auth_state,
 			       struct mailbox *mailbox);
 extern int mailbox_open_header_path(const char *name, const char *path,
-				    const char *acl, 
+				    const char *mpath, const char *acl, 
 				    struct auth_state *auth_state,
 				    struct mailbox *mailbox,
 				    int suppresslog);
 extern int mailbox_open_locked(const char *mbname,
 			       const char *mbpath,
+			       const char *metapath,
 			       const char *mbacl,
 			       struct auth_state *auth_state,
 			       struct mailbox *mb,
@@ -261,60 +297,53 @@ extern int mailbox_read_index_header(struct mailbox *mailbox);
 extern int mailbox_read_index_record(struct mailbox *mailbox,
 				     unsigned msgno,
 				     struct index_record *record);
-extern int mailbox_read_quota(struct quota *quota);
-extern void mailbox_hash_quota(char *buf, unsigned size, const char *qr);
 extern int mailbox_lock_header(struct mailbox *mailbox);
 extern int mailbox_lock_index(struct mailbox *mailbox);
 extern int mailbox_lock_pop(struct mailbox *mailbox);
-extern int mailbox_lock_quota(struct quota *quota);
 
 extern void mailbox_unlock_header(struct mailbox *mailbox);
 extern void mailbox_unlock_index(struct mailbox *mailbox);
 extern void mailbox_unlock_pop(struct mailbox *mailbox);
-extern void mailbox_unlock_quota(struct quota *quota);
 
 extern int mailbox_write_header(struct mailbox *mailbox);
 extern int mailbox_write_index_header(struct mailbox *mailbox);
+extern void mailbox_index_record_to_buf(struct index_record *record, char *buf);
 extern int mailbox_write_index_record(struct mailbox *mailbox,
 				      unsigned msgno,
 				      struct index_record *record, int sync);
 extern int mailbox_append_index(struct mailbox *mailbox,
 				struct index_record *record,
 				unsigned start, unsigned num, int sync);
-extern int mailbox_write_quota(struct quota *quota);
-
-extern int mailbox_delete_quota(struct quota *quota);
 
 extern int mailbox_expunge(struct mailbox *mailbox,
-			   int iscurrentdir,
-			   mailbox_decideproc_t *decideproc,
-			   void *deciderock);
-extern int mailbox_expungenews(struct mailbox *mailbox);
+			   mailbox_decideproc_t *decideproc, void *deciderock,
+			   int flags);
+extern int mailbox_cleanup(struct mailbox *mailbox, int iscurrentdir,
+			   mailbox_decideproc_t *decideproc, void *deciderock);
 
-extern int mailbox_findquota(char *ret, const char *name);
 extern void mailbox_make_uniqueid(char *name, unsigned long uidvalidity,
-				  char *uniqueid);
+				  char *uniqueid, size_t outlen);
 
-extern int mailbox_create(const char *name, char *path,
+extern int mailbox_create(const char *name, char *partition,
 			  const char *acl, const char *uniqueid, int format,
 			  struct mailbox *mailboxp);
 extern int mailbox_delete(struct mailbox *mailbox, int delete_quota_root);
 
 extern int mailbox_rename_copy(struct mailbox *oldmailbox, 
-			       const char *newname, char *newpath,
+			       const char *newname, char *newpartition,
 			       bit32 *olduidvalidityp, bit32 *newuidvalidityp,
 			       struct mailbox *mailboxp);
-extern int mailbox_rename_cleanup(struct mailbox *oldmailbox,
-				  int isinbox);
+extern int mailbox_rename_cleanup(struct mailbox *oldmailbox, int isinbox);
 
 extern int mailbox_sync(const char *oldname, const char *oldpath, 
-			const char *oldacl, 
-			const char *newname, char *newpath, 
+			const char *oldmpath, const char *oldacl, 
+			const char *newname, char *newpath, char *newmpath,
 			int docreate,
 			bit32 *olduidvalidityp, bit32 *newuidvalidtyp,
 			struct mailbox *mailboxp);
 
-extern int mailbox_copyfile(const char *from, const char *to);
-extern void mailbox_hash_mbox(char *buf, const char *root, const char *name);
+extern int mailbox_copyfile(const char *from, const char *to, int nolink);
+extern void mailbox_hash_mbox(char *buf, size_t buf_len,
+			      const char *root, const char *name);
 
 #endif /* INCLUDED_MAILBOX_H */
