@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.144.2.37 2005/12/13 19:36:08 murch Exp $
+ * $Id: pop3d.c,v 1.144.2.39 2006/02/01 19:25:38 murch Exp $
  */
 #include <config.h>
 
@@ -1291,7 +1291,7 @@ void cmd_capa()
 {
     int minpoll = config_getint(IMAPOPT_POPMINPOLL) * 60;
     int expire = config_getint(IMAPOPT_POPEXPIRETIME);
-    unsigned mechcount;
+    int mechcount;
     const char *mechlist;
 
     prot_printf(popd_out, "+OK List of capabilities follows\r\n");
@@ -1351,7 +1351,7 @@ void cmd_auth(char *arg)
      */
     if (!arg) {
 	const char *sasllist;
-	unsigned int mechnum;
+	int mechnum;
 
 	prot_printf(popd_out, "+OK List of supported mechanisms follows\r\n");
       
@@ -1694,6 +1694,10 @@ static void blat(int msg,int lines)
     if (buf[strlen(buf)-1] != '\n') prot_printf(popd_out, "\r\n");
 
     prot_printf(popd_out, ".\r\n");
+
+    /* Reset inactivity timer in case we spend a long time
+       pushing data to the client over a slow link. */
+    prot_resettimeout(popd_in);
 }
 
 static int parsenum(char **ptr)
