@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: reconstruct.c,v 1.87 2004/05/22 03:45:52 rjs3 Exp $ */
+/* $Id: reconstruct.c,v 1.81.2.16 2005/03/27 14:36:25 ken3 Exp $ */
 
 #include <config.h>
 
@@ -83,6 +83,7 @@
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
+#include "map.h"
 #include "message.h"
 #include "xmalloc.h"
 #include "global.h"
@@ -93,6 +94,7 @@
 #include "retry.h"
 #include "convert_code.h"
 #include "util.h"
+#include "byteorder64.h"
 
 extern int optind;
 extern char *optarg;
@@ -130,7 +132,7 @@ int main(int argc, char **argv)
     struct discovered head;
     char *alt_config = NULL;
     char *start_part = NULL;
-    const char *start_part_path = NULL;
+    const char *start_part_path = NULL, *start_part_mpath = NULL, *path;
 
     memset(&head, 0, sizeof(head));
 
@@ -138,7 +140,7 @@ int main(int argc, char **argv)
 
     /* Ensure we're up-to-date on the index file format */
     assert(INDEX_HEADER_SIZE == (OFFSET_SPARE2+4));
-    assert(INDEX_RECORD_SIZE == (OFFSET_CACHE_VERSION+4));
+    assert(INDEX_RECORD_SIZE == (OFFSET_MESSAGE_UUID+MESSAGE_UUID_PACKED_SIZE));
 
     while ((opt = getopt(argc, argv, "C:p:rmfx")) != EOF) {
 	switch (opt) {
@@ -186,6 +188,7 @@ int main(int argc, char **argv)
 	if (!start_part_path) {
 	    fatal(error_message(IMAP_PARTITION_UNKNOWN), EC_USAGE);
 	}
+	start_part_mpath = config_metapartitiondir(start_part);
     }
 
     if (mflag) {
@@ -208,7 +211,8 @@ int main(int argc, char **argv)
     if (start_part) {
 	/* We were handed a mailbox that does not exist currently */
 	if(optind == argc) {
-	    fprintf(stderr, "When using -p, you must specify a mailbox to attempt to reconstruct.");
+	    fprintf(stderr,
+		    "When using -p, you must specify a mailbox to attempt to reconstruct.");
 	    exit(EC_USAGE);
 	}
 
@@ -228,7 +232,7 @@ int main(int argc, char **argv)
 
 	    /* Does it exist */
 	    do {
-		r = mboxlist_lookup(buf, NULL, NULL, NULL);
+		r = mboxlist_lookup(buf, NULL, NULL);
 	    } while (r == IMAP_AGAIN);
 
 	    if(r != IMAP_MAILBOX_NONEXISTENT) {
@@ -239,8 +243,12 @@ int main(int argc, char **argv)
 	    }
 
 	    /* Does the suspected path *look* like a mailbox? */
-	    mailbox_hash_mbox(mbbuf, sizeof(mbbuf), start_part_path, buf);
-	    strlcat(mbbuf, "/cyrus.header", sizeof(mbbuf));
+	    path = (start_part_mpath &&
+		    (config_metapartition_files &
+		     IMAP_ENUM_METAPARTITION_FILES_HEADER)) ?
+		start_part_mpath : start_part_path;
+	    mailbox_hash_mbox(mbbuf, sizeof(mbbuf), path, buf);
+	    strlcat(mbbuf, "FNAME_HEADER", sizeof(mbbuf));
 	    if(stat(mbbuf, &sbuf) < 0) {
 		fprintf(stderr,
 			"%s does not appear to be a mailbox (no %s).\n",
@@ -415,7 +423,7 @@ int reconstruct(char *name, struct discovered *found)
 
     char *p;
 
-    char fnamebuf[MAX_MAILBOX_PATH+1];
+    char fnamebuf[MAX_MAILBOX_PATH+1], newfnamebuf[MAX_MAILBOX_PATH+1];
     FILE *newindex, *msgfile;
     DIR *dirp;
     struct dirent *dirent;
@@ -434,26 +442,34 @@ int reconstruct(char *name, struct discovered *found)
     char *list_acl, *list_part;
     int list_type;
 
-    unsigned long new_quota = 0;
+    uquota_t new_quota = 0;
 
     struct index_record message_index, old_index;
     static struct index_record zero_index;
+    struct body *body = NULL;
 
-    char *mypath, *myacl;
+    char *mypath, *mympath, *mypart, *myacl;
     int mytype;
-    char mbpath[MAX_MAILBOX_PATH+1];
+    char mbpath[MAX_MAILBOX_PATH+1], *path;
     
+    int expunge_fd = -1;
+
     /* Start by looking up current data in mailbox list */
-    r = mboxlist_detail(name, &mytype, &mypath, NULL, &myacl, NULL);
+    r = mboxlist_detail(name, &mytype, &mypath, &mympath,
+			&mypart, &myacl, NULL);
     if(r) return r;
     
     /* stat for header, if it is not there, we need to create it
      * note that we do not want to wind up with a fully-open mailbox,
      * so we will re-open. */
-    snprintf(mbpath, sizeof(mbpath), "%s%s", mypath, FNAME_HEADER);
+    path = (mympath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_HEADER)) ?
+	mympath : mypath;
+    snprintf(mbpath, sizeof(mbpath), "%s%s", path, FNAME_HEADER);
     if(stat(mbpath, &sbuf) == -1) {
 	/* Header doesn't exist, create it! */
-	r = mailbox_create(name, mypath, myacl, NULL,
+	r = mailbox_create(name, mypart, myacl, NULL,
 			   ((mytype & MBTYPE_NETNEWS) ?
 			    MAILBOX_FORMAT_NETNEWS :
 			    MAILBOX_FORMAT_NORMAL), NULL);
@@ -501,7 +517,8 @@ int reconstruct(char *name, struct discovered *found)
     r = mailbox_read_header_acl(&mailbox);
     if (r) return r;
 
-    r = mboxlist_detail(name, &list_type, NULL, &list_part, &list_acl, NULL);
+    r = mboxlist_detail(name, &list_type, NULL, NULL,
+			&list_part, &list_acl, NULL);
     if (r) return r;
 
     if(strcmp(list_acl, mailbox.acl)) {
@@ -527,7 +544,12 @@ int reconstruct(char *name, struct discovered *found)
     mailbox.pop3_last_login = 0;
 
     /* Create new index/cache files */
-    strlcpy(fnamebuf, FNAME_INDEX+1, sizeof(fnamebuf));
+    path = (mailbox.mpath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_INDEX)) ?
+	mailbox.mpath : mailbox.path;
+    strlcpy(fnamebuf, path, sizeof(fnamebuf));
+    strlcat(fnamebuf, FNAME_INDEX, sizeof(fnamebuf));
     strlcat(fnamebuf, ".NEW", sizeof(fnamebuf));
     newindex = fopen(fnamebuf, "w+");
     if (!newindex) {
@@ -535,7 +557,12 @@ int reconstruct(char *name, struct discovered *found)
 	return IMAP_IOERROR;
     }
 
-    strlcpy(fnamebuf, FNAME_CACHE+1, sizeof(fnamebuf));
+    path = (mailbox.mpath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_CACHE)) ?
+	mailbox.mpath : mailbox.path;
+    strlcpy(fnamebuf, path, sizeof(fnamebuf));
+    strlcat(fnamebuf, FNAME_CACHE, sizeof(fnamebuf));
     strlcat(fnamebuf, ".NEW", sizeof(fnamebuf));
     newcache_fd = open(fnamebuf, O_RDWR|O_TRUNC|O_CREAT, 0666);
     if (newcache_fd == -1) {
@@ -549,6 +576,65 @@ int reconstruct(char *name, struct discovered *found)
     fwrite(buf, 1, INDEX_HEADER_SIZE, newindex);
     retry_write(newcache_fd, buf, sizeof(bit32));
 
+    /* Remove any previously expunged messages */
+    /* XXX  For two-phase we will need to keep the messages referenced
+       in cyrus.expunge and refresh its header and offsets */
+    path = (mailbox.mpath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_EXPUNGE)) ?
+	mailbox.mpath : mailbox.path;
+    strlcpy(fnamebuf, path, sizeof(fnamebuf));
+    strlcat(fnamebuf, FNAME_EXPUNGE_INDEX, sizeof(fnamebuf));
+    if (stat(fnamebuf, &sbuf) != -1 &&
+	sbuf.st_size > OFFSET_LEAKED_CACHE &&
+	(expunge_fd = open(fnamebuf, O_RDONLY, 0666)) != -1) {
+	const char *index_base = NULL;
+	unsigned long index_len = 0;	/* mapped size */
+	int format, minor_version;
+	unsigned long start_offset, record_size, exists;
+
+	/* Map the file */
+	map_refresh(expunge_fd, 1, &index_base,
+		    &index_len, sbuf.st_size, "expunge", mailbox.name);
+
+	format = ntohl(*((bit32 *)(index_base+OFFSET_FORMAT)));
+	minor_version = ntohl(*((bit32 *)(index_base+OFFSET_MINOR_VERSION)));
+	start_offset = ntohl(*((bit32 *)(index_base+OFFSET_START_OFFSET)));
+	record_size = ntohl(*((bit32 *)(index_base+OFFSET_RECORD_SIZE)));
+	exists = ntohl(*((bit32 *)(index_base+OFFSET_EXISTS)));
+
+	/* Sanity check the header as best we can */
+	if (format == 0 && exists &&
+	    minor_version && minor_version <= MAILBOX_MINOR_VERSION &&
+	    start_offset && start_offset <= INDEX_HEADER_SIZE &&
+	    record_size && record_size <= INDEX_RECORD_SIZE &&
+	    sbuf.st_size >= (start_offset + exists * record_size)) {
+	    unsigned msgno;
+	    const char *p = index_base + start_offset;
+
+	    /* Delete message files */
+	    for (msgno = 1; msgno <= exists; msgno++, p += record_size) {
+		unsigned long uid = ntohl(*((bit32 *)(p + OFFSET_UID)));
+		char msgfname[MAILBOX_FNAME_LEN+1];
+
+		/* Sanity check UID */
+		if (uid == 0) {
+		    syslog(LOG_ERR, "IOERROR: %s zero expunge record %u/%lu",
+			   mailbox.name, msgno, exists);
+		    break;
+		}
+
+		mailbox_message_get_fname(&mailbox, uid,
+					  msgfname, sizeof(msgfname));
+		unlink(msgfname);
+	    }
+	}
+
+	map_free(&index_base, &index_len);
+	close(expunge_fd);
+    }
+    unlink(fnamebuf);
+
     /* Find all message files in directory */
     uid = (unsigned long *) xmalloc(UIDGROW * sizeof(unsigned long));
     uid_num = 0;
@@ -593,14 +679,19 @@ int reconstruct(char *name, struct discovered *found)
     mailbox.cache_fd = newcache_fd;
 
     for (msg = 0; msg < uid_num; msg++) {
-	char fnamebuf[MAILBOX_FNAME_LEN];
+	char msgfname[MAILBOX_FNAME_LEN+1];
 
 	message_index = zero_index;
 	message_index.uid = uid[msg];
 	
-	mailbox_message_get_fname(&mailbox, uid[msg], fnamebuf, sizeof(fnamebuf));
-	msgfile = fopen(fnamebuf, "r");
-	if (!msgfile) continue;
+	mailbox_message_get_fname(&mailbox, uid[msg], msgfname, sizeof(msgfname));
+	msgfile = fopen(msgfname, "r");
+	if (!msgfile) {
+	    fprintf(stderr, "reconstruct: fopen() failed for '%s' [error=%d] -- skipping.\n",
+		    msgfname, errno);
+	    continue;
+	}
+
 	if (fstat(fileno(msgfile), &sbuf)) {
 	    fclose(msgfile);
 	    continue;
@@ -608,7 +699,7 @@ int reconstruct(char *name, struct discovered *found)
 	if (sbuf.st_size == 0) {
 	    /* Zero-length message file--blow it away */
 	    fclose(msgfile);
-	    unlink(fnamebuf);
+	    unlink(msgfname);
 	    continue;
 	}
 
@@ -628,6 +719,8 @@ int reconstruct(char *name, struct discovered *found)
 		message_index.user_flags[i] =
 		  old_index.user_flags[i] & valid_user_flags[i];
 	    }
+            /* Copy across MessageUUID if confident that data on disk */
+            message_uuid_copy(&message_index.uuid, &old_index.uuid);
 	}
 	else {
 	    /* Message file write time is good estimate of internaldate */
@@ -635,10 +728,13 @@ int reconstruct(char *name, struct discovered *found)
 	    /* If we are recovering a message, assume new UIDL
 	       so that stupid clients will retrieve this message */
 	    mailbox.pop3_new_uidl = 1;
+            /* Wipe the Message UUID */
+            message_uuid_set_null(&message_index.uuid);
 	}
 	message_index.last_updated = time(0);
 	
-	if ((r = message_parse_file(msgfile, &mailbox, &message_index))!=0) {
+	if (((r = message_parse_file(msgfile, NULL, NULL, &body)) != 0) ||
+	    ((r = message_create_record(&mailbox, &message_index, body)) != 0)) {
 	    fclose(msgfile);
 	    fclose(newindex);
 	    mailbox_close(&mailbox);
@@ -646,6 +742,7 @@ int reconstruct(char *name, struct discovered *found)
 	    return r;
 	}
 	fclose(msgfile);
+	if (body) message_free_body(body);
 	
 	/* Write out new entry in index file */
 	mailbox_index_record_to_buf(&message_index, buf);
@@ -664,6 +761,8 @@ int reconstruct(char *name, struct discovered *found)
 	if (message_index.system_flags & FLAG_DELETED) new_deleted++;
 	new_quota += message_index.size;
     }
+
+    if (body) free(body);
     
     /* Write out new index file header */
     rewind(newindex);
@@ -685,7 +784,16 @@ int reconstruct(char *name, struct discovered *found)
     *((bit32 *)(buf+OFFSET_EXISTS)) = htonl(new_exists);
     *((bit32 *)(buf+OFFSET_LAST_APPENDDATE)) = htonl(mailbox.last_appenddate);
     *((bit32 *)(buf+OFFSET_LAST_UID)) = htonl(mailbox.last_uid);
+
+    /* quotas may be 64bit now */
+#ifdef HAVE_LONG_LONG_INT
+    *((bit64 *)(buf+OFFSET_QUOTA_MAILBOX_USED64)) = htonll(new_quota);
+#else
+    /* zero the unused 32bits */
+    *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED64)) = htonl(0);
     *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)) = htonl(new_quota);
+#endif
+
     *((bit32 *)(buf+OFFSET_POP3_LAST_LOGIN)) = htonl(mailbox.pop3_last_login);
     *((bit32 *)(buf+OFFSET_UIDVALIDITY)) = htonl(mailbox.uidvalidity);
     *((bit32 *)(buf+OFFSET_DELETED)) = htonl(new_deleted);
@@ -694,6 +802,8 @@ int reconstruct(char *name, struct discovered *found)
     *((bit32 *)(buf+OFFSET_POP3_NEW_UIDL)) = htonl(mailbox.pop3_new_uidl);
     *((bit32 *)(buf+OFFSET_LEAKED_CACHE)) = htonl(0);
 
+    message_uuid_pack(&message_index.uuid, buf+OFFSET_MESSAGE_UUID);
+
     n = fwrite(buf, 1, INDEX_HEADER_SIZE, newindex);
     fflush(newindex);
     if (n != INDEX_HEADER_SIZE || ferror(newindex) 
@@ -722,16 +832,28 @@ int reconstruct(char *name, struct discovered *found)
     }
 
     /* Rename new index/cache file in place */
-    strlcpy(fnamebuf, FNAME_INDEX+1, sizeof(fnamebuf));
-    strlcat(fnamebuf, ".NEW", sizeof(fnamebuf));
-    if (rename(fnamebuf, FNAME_INDEX+1)) {
+    path = (mailbox.mpath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_INDEX)) ?
+	mailbox.mpath : mailbox.path;
+    strlcpy(fnamebuf, path, sizeof(fnamebuf));
+    strlcat(fnamebuf, FNAME_INDEX, sizeof(fnamebuf));
+    strlcpy(newfnamebuf, fnamebuf, sizeof(newfnamebuf));
+    strlcat(newfnamebuf, ".NEW", sizeof(fnamebuf));
+    if (rename(newfnamebuf, fnamebuf)) {
 	fclose(newindex);
 	mailbox_close(&mailbox);
 	return IMAP_IOERROR;
     }
-    strlcpy(fnamebuf, FNAME_CACHE+1, sizeof(fnamebuf));
-    strlcat(fnamebuf, ".NEW", sizeof(fnamebuf));
-    if (rename(fnamebuf, FNAME_CACHE+1)) {
+    path = (mailbox.mpath &&
+	    (config_metapartition_files &
+	     IMAP_ENUM_METAPARTITION_FILES_CACHE)) ?
+	mailbox.mpath : mailbox.path;
+    strlcpy(fnamebuf, path, sizeof(fnamebuf));
+    strlcat(fnamebuf, FNAME_CACHE, sizeof(fnamebuf));
+    strlcpy(newfnamebuf, fnamebuf, sizeof(newfnamebuf));
+    strlcat(newfnamebuf, ".NEW", sizeof(newfnamebuf));
+    if (rename(newfnamebuf, fnamebuf)) {
 	fclose(newindex);
 	mailbox_close(&mailbox);
 	return IMAP_IOERROR;
@@ -742,6 +864,12 @@ int reconstruct(char *name, struct discovered *found)
     mailbox_close(&mailbox);
 
     if (found) {
+	if (mympath &&
+	    (config_metapartition_files & IMAP_ENUM_METAPARTITION_FILES_HEADER) &&
+	    chdir(mympath) == -1) {
+	    return IMAP_IOERROR;
+	}
+
 	/* we recurse down this directory to see if there's any mailboxes
 	   under this not in the mailboxes database */
 	dirp = opendir(".");
@@ -756,8 +884,8 @@ int reconstruct(char *name, struct discovered *found)
 
 	    /* ok, we found a directory that doesn't have a dot in it;
 	     is there a cyrus.header file? */
-	    snprintf(fnamebuf, sizeof(fnamebuf), "%s/cyrus.header",
-		     dirent->d_name);
+	    snprintf(fnamebuf, sizeof(fnamebuf), "%s%s",
+		     dirent->d_name, FNAME_HEADER);
 	    if (stat(fnamebuf, &sbuf) < 0) continue;
 
 	    /* ok, we have a real mailbox directory */
@@ -766,7 +894,7 @@ int reconstruct(char *name, struct discovered *found)
 
 	    /* does fnamebuf exist as a mailbox in mboxlist? */
 	    do {
-		r = mboxlist_lookup(fnamebuf, NULL, NULL, NULL);
+		r = mboxlist_lookup(fnamebuf, NULL, NULL);
 	    } while (r == IMAP_AGAIN);
 	    if (!r) continue; /* mailbox exists; it'll be reconstructed
 			         with a -r */
@@ -786,6 +914,8 @@ int reconstruct(char *name, struct discovered *found)
     return r;
 }
 
+/* XXX  What is the stuff below used for?  Do we need to metadata-ize it? */
+
 /* List of directories to scan for mailboxes */
 struct todo {
     char *name;
