@@ -1,7 +1,7 @@
 /* imtest.c -- IMAP/POP3/NNTP/LMTP/SMTP/MUPDATE/MANAGESIEVE test client
  * Ken Murchison (multi-protocol implementation)
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.107 2006/01/20 20:31:23 jeaton Exp $
+ * $Id: imtest.c,v 1.93.2.13 2005/10/28 14:51:19 ken3 Exp $
  *
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -1282,10 +1282,7 @@ static void interactive(struct protocol_t *protocol, char *filename)
     FD_SET(fd_out, &write_set);
     FD_SET(sock, &write_set);
     
-    nfds = fd;
-    if (nfds < sock) nfds = sock;
-    if (nfds < fd_out) nfds = fd_out;
-    nfds++;
+    nfds = getdtablesize();
     
     if (filename != NULL) {
 	donewritingfile = 0;
@@ -2244,6 +2241,13 @@ static struct protocol_t protocols[] = {
       { "AUTHENTICATE", INT_MAX, 1, "OK", "NO", NULL, "*", &sieve_parse_success },
       NULL, { "LOGOUT", "OK" }, NULL, NULL, NULL
     },
+    { "csync", NULL, "csync",
+      { 1, "* OK", NULL },
+      { NULL , "* OK", "* STARTTLS", "* SASL ", NULL },
+      { "STARTTLS", "OK", "NO", 1 },
+      { "AUTHENTICATE", INT_MAX, 0, "OK", "NO", "+ ", "*", NULL },
+      NULL, { "EXIT", "OK" }, NULL, NULL, NULL
+    },
     { NULL, NULL, NULL,
       { 0, NULL, NULL },
       { NULL, NULL, NULL, NULL, NULL },
@@ -2421,6 +2425,8 @@ int main(int argc, char **argv)
 	    prot = "mupdate";
 	else if (!strcasecmp(prog, "sivtest"))
 	    prot = "sieve";
+	else if (!strcasecmp(prog, "synctest"))
+	    prot = "csync";
     }
 
     protocol = protocols;
