@@ -1,10 +1,9 @@
 /* actions.c -- executes the commands for timsieved
  * Tim Martin
- * $Id: actions.c,v 1.31 2002/10/18 20:18:01 ken3 Exp $
- * 
+ * $Id: actions.c,v 1.36.2.5 2005/11/18 14:16:11 murch Exp $
  */
 /*
- * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -55,16 +54,20 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <sys/param.h>
+#include <sys/types.h>
+#include <fcntl.h>
 #include <syslog.h>
 #include <dirent.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
+#include <retry.h>
 
 #include "prot.h"
 #include "tls.h"
 #include "util.h"
-#include "imapconf.h"
+#include "global.h"
+#include "libconfig.h"
 #include "xmalloc.h"
 #include "sieve_interface.h"
 
@@ -72,19 +75,24 @@
 #include "actions.h"
 #include "scripttest.h"
 
+#include "sync_log.h"
+
 /* after a user has authentication, our current directory is their Sieve 
    directory! */
 
+extern int sieved_userisadmin;
 char *sieve_dir = NULL;
 
+const static char *sieved_userid = NULL;
+
 int actions_init(void)
 {
   int sieve_usehomedir = 0;
 
-  sieve_usehomedir = config_getswitch("sieveusehomedir", 0);
+  sieve_usehomedir = config_getswitch(IMAPOPT_SIEVEUSEHOMEDIR);
   
   if (!sieve_usehomedir) {
-      sieve_dir = (char *) config_getstring("sievedir", "/usr/sieve");
+      sieve_dir = (char *) config_getstring(IMAPOPT_SIEVEDIR);
   } else {
       /* can't use home directories with timsieved */
       syslog(LOG_ERR, "can't use home directories");
@@ -97,19 +105,42 @@ int actions_init(void)
 
 int actions_setuser(const char *userid)
 {
-  char hash;
-  char *foo=sieve_dir;
+  char userbuf[1024], *user, *domain = NULL;
+  char *foo = sieve_dir;
+  size_t size = 1024, len;
   int result;  
 
-  sieve_dir=(char *) xmalloc(1024);
+  sieve_dir = (char *) xzmalloc(size+1);
   
-  hash = (char) dir_hash_c(userid);
-    
-  snprintf(sieve_dir, 1023, "%s/%c/%s", foo, hash,userid);
+  sieved_userid = xstrdup(userid);
+  user = (char *) userid;
+  if (config_virtdomains && strchr(user, '@')) {
+      /* split the user and domain */
+      strlcpy(userbuf, userid, sizeof(userbuf));
+      user = userbuf;
+      if ((domain = strrchr(user, '@'))) *domain++ = '\0';
+  }
+
+  len = strlcpy(sieve_dir, foo, size);
+
+  if (domain) {
+      char dhash = (char) dir_hash_c(domain);
+      len += snprintf(sieve_dir+len, size-len, "%s%c/%s",
+		      FNAME_DOMAINDIR, dhash, domain);
+  }
+
+  if (sieved_userisadmin) {
+      strlcat(sieve_dir, "/global", size);
+  }
+  else {
+      char hash = (char) dir_hash_c(user);
+      snprintf(sieve_dir+len, size-len, "/%c/%s", hash, user);
+  }
 
   result = chdir(sieve_dir);
   if (result != 0) {
-      result = mkdir(sieve_dir, 0755);
+      result = cyrus_mkdir(sieve_dir, 0755);
+      if (!result) result = mkdir(sieve_dir, 0755);
       if (!result) result = chdir(sieve_dir);
       if (result) {
 	  syslog(LOG_ERR, "mkdir %s: %m", sieve_dir);
@@ -145,17 +176,19 @@ int scriptname_valid(mystring_t *name)
   return TIMSIEVE_OK;
 }
 
-int capabilities(struct protstream *conn, sasl_conn_t *saslconn)
+int capabilities(struct protstream *conn, sasl_conn_t *saslconn,
+		 int starttls_done, int authenticated)
 {
     const char *sasllist;
-    unsigned mechcount;
+    int mechcount;
 
     /* implementation */
     prot_printf(conn, "\"IMPLEMENTATION\" \"Cyrus timsieved %s\"\r\n",
 		CYRUS_VERSION);
     
     /* SASL */
-    if (sasl_listmech(saslconn, NULL, 
+    if (!authenticated &&
+	sasl_listmech(saslconn, NULL, 
 		    "\"SASL\" \"", " ", "\"\r\n",
 		    &sasllist,
 		    NULL, &mechcount) == SASL_OK && mechcount > 0)
@@ -164,9 +197,9 @@ int capabilities(struct protstream *conn, sasl_conn_t *saslconn)
     }
     
     /* Sieve capabilities */
-    prot_printf(conn,"\"SIEVE\" \"%s\"\r\n",sieve_listextensions());
+    prot_printf(conn,"\"SIEVE\" \"%s\"\r\n",sieve_listextensions(interp));
 
-    if (tls_enabled("sieve")) {
+    if (tls_enabled() && !starttls_done && !authenticated) {
 	prot_printf(conn, "\"STARTTLS\"\r\n");
     }
 
@@ -279,7 +312,9 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
   int lup;
   int result;
   char path[1024], p2[1024];
+  char bc_path[1024], bc_p2[1024];
   int maxscripts;
+  sieve_script_t *s;
 
   result = scriptname_valid(name);
   if (result!=TIMSIEVE_OK)
@@ -293,7 +328,7 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
 
   else {
       /* see if this would put the user over quota */
-      maxscripts = config_getint("sieve_maxscripts",5);
+      maxscripts = config_getint(IMAPOPT_SIEVE_MAXSCRIPTS);
 
       if (countscripts(string_DATAPTR(name))+1 > maxscripts)
       {
@@ -308,6 +343,7 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
       stream = fopen(path, "w+");
   }
 
+
   if (stream == NULL) {
       prot_printf(conn, "NO \"Unable to open script for writing (%s)\"\r\n",
 		  path);
@@ -330,7 +366,7 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
   /* let's make sure this is a valid script
      (no parse errors)
   */
-  result = is_script_parsable(stream, &errstr);
+  result = is_script_parsable(stream, &errstr, &s);
 
   if (result != TIMSIEVE_OK) {
       if (errstr && *errstr) { 
@@ -349,11 +385,54 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
   fclose(stream);
   
   if (!verify_only) {
+      int fd;
+      bytecode_info_t *bc;
+      
+      /* Now, generate the bytecode */
+      if(sieve_generate_bytecode(&bc, s) == -1) {
+	  unlink(path);
+	  sieve_script_free(&s);
+	  prot_printf(conn, "NO \"bytecode generate failed\"\r\n");
+	  return TIMSIEVE_FAIL;
+      }
+
+      /* Now, open the new file */
+      snprintf(bc_path, 1023, "%s.bc.NEW", string_DATAPTR(name));
+      fd = open(bc_path, O_CREAT | O_TRUNC | O_WRONLY, 0600);
+      if(fd < 0) {
+	  unlink(path);
+	  sieve_free_bytecode(&bc);
+	  sieve_script_free(&s);
+	  prot_printf(conn, "NO \"couldn't open bytecode file\"\r\n");
+	  return TIMSIEVE_FAIL;
+      }
+	  
+      /* Now, emit the bytecode */
+      if(sieve_emit_bytecode(fd, bc) == -1) {
+	  close(fd);
+	  unlink(path);
+	  unlink(bc_path);
+	  sieve_free_bytecode(&bc);
+	  sieve_script_free(&s);
+	  prot_printf(conn, "NO \"bytecode emit failed\"\r\n");
+	  return TIMSIEVE_FAIL;
+      }
+
+      sieve_free_bytecode(&bc);
+      sieve_script_free(&s);
+
+      close(fd);
+
+      /* Now, rename! */
       snprintf(p2, 1023, "%s.script", string_DATAPTR(name));
+      snprintf(bc_p2, 1023, "%s.bc", string_DATAPTR(name));
       rename(path, p2);
+      rename(bc_path, bc_p2);
+
   }
 
   prot_printf(conn, "OK\r\n");
+  sync_log_sieve(sieved_userid);
 
   return TIMSIEVE_OK;
 }
@@ -362,10 +441,11 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
 
 static int deleteactive(struct protstream *conn)
 {
-    if (unlink("default") != 0) {
+    if (unlink("defaultbc") != 0) {
 	prot_printf(conn,"NO \"Unable to unlink active script\"\r\n");
 	return TIMSIEVE_FAIL;
     }
+    sync_log_sieve(sieved_userid);
 
     return TIMSIEVE_OK;
 }
@@ -377,12 +457,12 @@ static int isactive(char *name)
     char filename[1024];
     char activelink[1024];
 
-    snprintf(filename, 1023, "%s.script", name);
+    snprintf(filename, 1023, "%s.bc", name);
     memset(activelink, 0, sizeof(activelink));
-    if ((readlink("default", activelink, sizeof(activelink)-1) < 0) && 
+    if ((readlink("defaultbc", activelink, sizeof(activelink)-1) < 0) && 
 	(errno != ENOENT)) 
     {
-	syslog(LOG_ERR, "readlink(default): %m");
+	syslog(LOG_ERR, "readlink(defaultbc): %m");
 	return FALSE;
     }
 
@@ -419,6 +499,16 @@ int deletescript(struct protstream *conn, mystring_t *name)
       return TIMSIEVE_FAIL;
   }
 
+  snprintf(path, 1023, "%s.bc", string_DATAPTR(name));
+
+  result = unlink(path);
+
+  if (result != 0) {
+      prot_printf(conn,"NO \"Error deleting bytecode\"\r\n");
+      return TIMSIEVE_FAIL;
+  }
+  sync_log_sieve(sieved_userid);
+
   prot_printf(conn,"OK\r\n");
   return TIMSIEVE_OK;
 }
@@ -521,26 +611,27 @@ int setactive(struct protstream *conn, mystring_t *name)
     }
 
     /* get the name of the active sieve script */
-    snprintf(filename, sizeof filename, "%s.script", string_DATAPTR(name));
+    snprintf(filename, sizeof(filename), "%s.bc", string_DATAPTR(name));
 
     /* ok we want to do this atomically so let's
        - make <activesieve>.NEW as a hard link
        - rename it to <activesieve>
     */
-    result = symlink(filename, "default.NEW");
+    result = symlink(filename, "defaultbc.NEW");
     if (result) {
-	syslog(LOG_ERR, "symlink(%s, default.NEW): %m", filename);
+	syslog(LOG_ERR, "symlink(%s, defaultbc.NEW): %m", filename);
 	prot_printf(conn, "NO \"Can't make link\"\r\n");    
 	return TIMSIEVE_FAIL;
     }
 
-    result = rename("default.NEW", "default");
+    result = rename("defaultbc.NEW", "defaultbc");
     if (result) {
-	unlink("default.NEW");
-	syslog(LOG_ERR, "rename(default.NEW, default): %m");
+	unlink("defaultbc.NEW");
+	syslog(LOG_ERR, "rename(defaultbc.NEW, defaultbc): %m");
 	prot_printf(conn,"NO \"Error renaming\"\r\n");
 	return TIMSIEVE_FAIL;
     }
+    sync_log_sieve(sieved_userid);
 
     prot_printf(conn,"OK\r\n");
     return TIMSIEVE_OK;
@@ -560,7 +651,7 @@ int cmd_havespace(struct protstream *conn, mystring_t *sieve_name, unsigned long
     }
 
     /* see if the size of the script is too big */
-    maxscriptsize = config_getint("sieve_maxscriptsize", 32);
+    maxscriptsize = config_getint(IMAPOPT_SIEVE_MAXSCRIPTSIZE);
     maxscriptsize *= 1024;
 
     if (num > maxscriptsize)
@@ -573,7 +664,7 @@ int cmd_havespace(struct protstream *conn, mystring_t *sieve_name, unsigned long
     }
 
     /* see if this would put the user over quota */
-    maxscripts = config_getint("sieve_maxscripts",5);
+    maxscripts = config_getint(IMAPOPT_SIEVE_MAXSCRIPTS);
 
     if (countscripts(string_DATAPTR(sieve_name))+1 > maxscripts)
     {
